/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 3    BMP2BIG.C    31-Mar-93    13:14:52 #    */
/* ******************************************************* */
/*    METAGRAPHICS SOFTWARE CORPORATION (c) 1993           */
/*    Demonstrates panning a large file in a small display */
/* ******************************************************* */

/* 
   Note:
   In order to keep this sample as simple as possible, it requires;

            EGA32x200 mode
            Mouse Driver
            a 16 color BMP file

   Display speed can be improved by using GetBMP or reading to 
   an offscreen bitmap.

*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>

#include "metawndo.h"   /* master MetaWINDOW include file */
#include "bmplab.h"     /* BMP lab include file */


/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /* stack size 14K */
#endif


void main( int argc, char *argv[] )
{
   int      i, x, y, buttons, oldx, oldy;
   rect     scrnR, srcR;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   palData  RGBInfo[256];     /* buffer for reading the files palette    */

   /* check for a file name on the command line */
   if( argc < 2 ) {
      printf("BMP2BIG - Display BMP file larger than screen\n");
      printf("Usage: BMP2BIG filename\n");
      printf("Use mouse to control panning\n");
      exit(1);
   }

   /* hard code display type to EGA 320x200 */

   /* init the system */
   i = InitGraphics( EGA320x200 );
   if( i != 0 ) {
      printf("MetaWINDOW Initialization error #%d\n", i);
      exit(2);
   }

   /* Read BMP File Header Information */
   if ( ReadBMPInfo( argv[1], &bmpHeader, RGBInfo ) == False )  {
      printf("%s is not a valid BMP file\n", argv[1]);
      StopGraphics();
      exit(2);
   }

   /* put into graphics mode */
   SetDisplay( GrafPg0 ); 

   /* start up the mouse */
   i = InitMouse( MsDriver );
   if( i != 0 ) {
      SetDisplay( TextPg0 );
      printf("Program Requires a mouse driver\n");
      StopGraphics();
      exit(2);
   }

   /* set mouse to return limits of bmp file */
   LimitMouse( 0, 0, bmpHeader.width, bmpHeader.height );

   /* set palette, based on palette in BMP file */
   WritePalette( 0, 0, (int) QueryColors(), RGBInfo );

   /* get dimensions of screen */
   ScreenRect( &scrnR );

   oldx = oldy = 0;

   do {

      /* ask mouse where it is */
      ReadMouse( &x, &y, &buttons );

      /* if no change, do nothing */
      if( x == oldx && y == oldy )
         continue;

      /* mouse moved, record new position */
      oldx = x; oldy = y;

      /* position to section of bmp file */
      DupRect( &scrnR, &srcR );
      OffsetRect( &srcR, x, y );

      /* keep source rect within bmp image limits */
      if( srcR.Xmax > bmpHeader.width ) {
         srcR.Xmax = bmpHeader.width;
         srcR.Xmin = srcR.Xmax - ( scrnR.Xmax - scrnR.Xmin );
      }
      if( srcR.Ymax > bmpHeader.height ) {
         srcR.Ymax = bmpHeader.height;
         srcR.Ymin = srcR.Ymax - ( scrnR.Ymax - scrnR.Ymin );
      }

      /* Read section of BMP File to the screen */

      i = ReadBMP( argv[1], &srcR, NULL );
      if( i == False )  {
         printf("BMP file not compatible with display mode\n");
         StopGraphics();
         exit(4);
      }

   } while ( buttons == 0 );



   SetDisplay( TextPg0 );
   StopGraphics();
   i = QueryError();
   printf("QueryError=%d/%d\n",i >> 7,i & 127);
   exit(i);

}


