/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 4    BMPINCR.C    31-Mar-93    13:14:16 #    */
/* ************************************************** */
/*    METAGRAPHICS SOFTWARE CORPORATION (c) 1993      */
/*    Demonstrates use of incremental BMP  operations */
/*    Functions: OpenBMP(), NextBMP(), and CloseBMP() */
/* ************************************************** */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>

#include "metawndo.h"   /* master MetaWINDOW include file */
#include "bmplab.h"     /* BMP lab include file */


/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /* stack size 14K */
#endif

/* global vaiables */
int   GrafixCard,GrafixInput;


void main( int argc, char *argv[] )
{
   int      i, height,y;
   rect     scrnR;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   palData  RGBInfo[256];     /* buffer for reading the files palette    */
   unsigned  imsize;          /* size of required image buffer           */
   image     *myimage;        /* pointer to allocated image buffer       */
   grafPort  *myport;         /* pointer to current port                 */     
   rect      dstR;            /* rectangle used to WriteImage() raster   */

   /* check for a file name on the command line */
   if( argc < 2 ) {
      printf("BMPINCR - Read BMP fileincrementally \n");
      printf("Usage: BMPINCR filename\n");
      exit(1);
   }

   /* let user select display type */
   MetQuery(argc,argv);      

   /* init the system */
   i = InitGraphics( GrafixCard );
   if( i != 0 ) {
      printf("MetaWINDOW Initialization error #%d\n", i);
      exit(2);
   }


   /* Open the BMP File and read  header information */
   imsize =  OpenBMP( argv[1], &bmpHeader, RGBInfo );
   if ( imsize == 0)  {
      printf("%s is not a valid BMP file\n", argv[1]);
      StopGraphics();
      exit(3);
   }

   /* check that this file will display in current video mode */ 
   GetPort( &myport );

   if( bmpHeader.dspColors != bmpHeader.bmpColors ) {
      printf("files does not match current display\n");
      StopGraphics();
      exit(4);
   }

   /* allocate an image buffer large enough for one raster of the bmp file */
   myimage = malloc( imsize );
   if( myimage == NULL ) {
      printf("cant allocate image buffer of %u bytes\n", imsize );
      StopGraphics();
      exit(5);
   }

   /* put into graphics mode */
   SetDisplay( GrafPg0 ); 

   /* clear the screen */
   ScreenRect( &dstR );
   EraseRect( &dstR );

   /* set palette, based on palette in BMP file */
   WritePalette( 0, 0, (int) QueryColors(), RGBInfo );

   /* set up a rectangle for the destination WriteImage() */
   dstR.Ymin = dstR.Ymax - 1;

   /* compute number of rasters in the bmp  file */
   height = bmpHeader.height;


   /* process each bmp file raster individually */
   for( y = height; y > 0; y-- ) {

      /* read next raster into an image buffer */
      i = NextBMP( &bmpHeader, myimage );

      if( i == False )  {
         MoveTo( 0, 100 );
         DrawString("Error reading BMP file!" );
         break;
      }

      /*  at this point image is available for custom functions */

      /* put the image on the current bitmap */
      WriteImage( &dstR, myimage );

      /* advance destination rectangle to next raster*/
      OffsetRect( &dstR, 0, -1 );

   } /* for Y */

   /*  wait for a key press */
   getch();

   /* close the BMP file opened via OpenBMP */
   CloseBMP( &bmpHeader );

   /* free the image buffer */
   free( myimage );


   SetDisplay( TextPg0 );
   StopGraphics();
   i = QueryError();
   printf("QueryError=%d/%d\n",i >> 7,i & 127);
   exit(i);

}

#include "metquery.c"

