/*  %kw # %v    %n    %d    %t #                    */
/*  Version # 3    BMPINFO.C    4-Mar-93    10:20:20 #    */
/* ************************************************ */
/*  METAGRAPHICS SOFTWARE CORPORATION (c) 1993      */
/*  Demonstrates usage of ReadBMPInfo               */
/* ************************************************ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>

#include "metawndo.h"      /* master MetaWINDOW include file */
#include "bmplab.h"        /* BMP-lab include file           */


/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /* stack size 14K */
#endif


/* function prototypes */
void draw_info( bmpHead *header );
void draw_palette( rect *pR );

/* global variables */
int  GrafixCard,GrafixInput;


void main( int argc, char *argv[] )
{
   int      i;
   rect     scrnR;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   palData  RGBInfo[256];     /* buffer for reading the files palette    */
   dirRec   fileDir;          /* buffer for file directory information   */

   /* check for a file name on the command line */
   if( argc < 2 ) {
      printf("BMPINFO - Display BMP file information\n");
      printf("Usage: BMPINFO filename\n");
      exit(1);
   }

   /* let user select display type */
   MetQuery(argc,argv);      

   /* init the system */
   i = InitGraphics( GrafixCard );
   if( i != 0 ) {
      printf("MetaWINDOW Initialization error #%d\n", i);
      exit(2);
   }
   
   /* check if file exists */
   i = FileQuery( argv[1], &fileDir, 1 );
   if( i != 1 ) {
      printf("%s does not exist!\n", argv[1] );
      exit(3);
   }

   /* put into graphics mode */
   SetDisplay( GrafPg0 ); 

   /* set a virtual coordinate system of 1000x1000 */
   SetRect( &scrnR, 0, 0, 1000, 1000 );
   VirtualRect( &scrnR );

   /* clear the screen */
   EraseRect( &scrnR );

   /* print the file name */
   MoveTo( 0, 50 );
   DrawString("File: "); DrawString( argv[1] );

   /* clear palette array to all white  */
   for( i = 0; i < 256; i++ ) {
      RGBInfo[i].palRed = -1;
      RGBInfo[i].palGreen = -1;
      RGBInfo[i].palBlue = -1;
   }

   /* read the info from the BMP file */
   i = ReadBMPInfo( argv[1], &bmpHeader, RGBInfo );

   /* display a warning if BMP-Lab doesn't think it's a valid file */
   if( i == False ) {

      MoveTo( 0, 150 );
      PenColor( Red );
      DrawString("WARNING: Not a valid BMP file!");

      /* keep palette 0 black, so text will still show up */
      RGBInfo[0].palRed = 0;
      RGBInfo[0].palGreen = 0;
      RGBInfo[0].palBlue = 0;

   }

   /* draw the header info */
   MoveTo( 0, 200  );
   draw_info( &bmpHeader );

   /*
   wait for a key before loading palette in case palette
   settings makes header text too hard to read                
   */
   MoveTo( 700, 50 );
   TextAlign( alignCenter, alignBaseline );
   DrawString("Press a key to see palette");
   getch();

   /* set palette, based on palette in BMP file */
   WritePalette( 0, 0, (int) QueryColors(), RGBInfo );

   /* draw the palette */
   SetRect( &scrnR, 400, 0, 1000, 1000 );
   draw_palette( &scrnR );

   /* wait for a key press */
   getch();

   SetDisplay(TextPg0);
   StopGraphics();
   i=QueryError();
   printf("\nQueryError=%d/%d",i >> 7,i & 127);
   exit(i);
}

/*
Print out the header information
*/

void draw_info( bmpHead *header )
{
   int x,y;
   char buf[80];

#define lnSpace 50

   PenColor( White );
   QueryPosn( &x, &y);

   DrawString( buf ); y += lnSpace; MoveTo( x, y );
   sprintf(buf, "Encoding type   : %d", header->rle );
   DrawString( buf ); y += lnSpace; MoveTo( x, y );
   sprintf(buf, "Resolution      : %d x %d", header->xRes,
                                           header->yRes );
   DrawString( buf ); y += lnSpace; MoveTo( x, y );

   sprintf(buf, "Bits per pixel  : %d", header->bits );
   DrawString( buf ); y += lnSpace; MoveTo( x, y );
   sprintf(buf, "Planes per pixel: %d", header->planes);   
   DrawString( buf ); y += lnSpace; MoveTo( x, y );
   sprintf(buf, "Bytes per line  : %d", header->rowBytes);   
   DrawString( buf ); y += lnSpace; MoveTo( x, y );
   DrawString( buf ); y += lnSpace; MoveTo( x, y );
   sprintf(buf, "Screen Size     : %d x %d ",header->width,
                                             header->height );
   DrawString( buf ); y += lnSpace; MoveTo( x, y );


}


/*
Draw a screen full of boxes, one for each color the current display mode
is capable of. 
*/
void draw_palette( rect *pR )
{
   int color, max_colors;
   int deltaX, deltaY, sizeX, sizeY;
   rect r;

   /* clear the box */
   EraseRect( pR );

   /* find the maximum color index for the current bitmap */
   max_colors = (int) QueryColors();

   /* size the boxes */
   deltaX = 40;
   deltaY = 50;
   sizeX = 30;
   sizeY = 40;

   SetRect( &r, pR->Xmin, pR->Ymin, pR->Xmin + sizeX, pR->Ymin + sizeY );

   for( color = 0; color <= max_colors; color++ ) {

      PenColor( color );
      PaintRect( &r );
      PenColor( White );
      FrameRect( &r );

      OffsetRect( &r, deltaX, 0 );

      if( r.Xmax > pR->Xmax ) {
         r.Xmin = pR->Xmin; r.Xmax = pR->Xmin + sizeX;
         OffsetRect( &r, 0, deltaY );
      }

   }

}

#include "metquery.c"


