/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 3    BMPOFFMP.C    31-Mar-93    13:13:22 #    */
/* ******************************************************* */
/*    METAGRAPHICS SOFTWARE CORPORATION (c) 1993           */
/*    Demonstrates use of ReadBMP() to an offscreen bitmap */
/* ******************************************************* */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>

#include "metawndo.h"   /* master MetaWINDOW include file */
#include "bmplab.h"     /* BMP lab include file */


/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /* stack size 14K */
#endif

/* global vaiables */
int   GrafixCard,GrafixInput;


void main( int argc, char *argv[] )
{
   int      i;
   rect     scrnR;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   palData  RGBInfo[256];     /* buffer for reading the files palette    */

   grafMap  offscrMap,  *defaultMap;
   grafPort offscrPort, *defaultPort;

   /* check for a file name on the command line */
   if( argc < 2 ) {
      printf("BMPOFFMP - Display BMP file from offscreen bitmap\n");
      printf("Usage: BMPOFFMP filename\n");
      exit(1);
   }

   /* let user select display type */
   MetQuery(argc,argv);      

   /* init the system */
   i = InitGraphics( GrafixCard );
   if( i != 0 ) {
      printf("MetaWINDOW Initialization error #%d\n", i);
      exit(2);
   }

   /* Read BMP File Header Information */
   if ( ReadBMPInfo( argv[1], &bmpHeader, RGBInfo ) == False )  {
      printf("%s is not a valid BMP file\n", argv[1]);
      StopGraphics();
      exit(2);
   }


   /* launch an offscreen bitmap similar to the current bitmap */

   /* get pointer to the current port and grafMap */
   GetPort( &defaultPort );
   defaultMap = defaultPort->portMap; 

   /* make offscreen bitmap same dimesions as current bitmap */
   offscrMap.pixWidth  = defaultMap->pixWidth;  
   offscrMap.pixHeight = defaultMap->pixHeight; 
   offscrMap.pixBits   = defaultMap->pixBits;   
   offscrMap.pixPlanes = defaultMap->pixPlanes; 
   offscrMap.pixResX   = defaultMap->pixResX; 
   offscrMap.pixResY   = defaultMap->pixResY; 

   /* allocate the offscreen bitmap in regular memory */
   if( InitBitmap( cMEMORY, &offscrMap ) )  {
      CloseBitmap( &offscrMap );
      printf("Unable to launch offscreen system\n");
      StopGraphics();
      exit(3);
   }

   /* make identical copy of current port */
   memcpy( &offscrPort, defaultPort, sizeof( grafPort ) ); 

   /* make new port current port */
   SetPort( &offscrPort );

   /* assign ofscreen bitmap to the port */
   PortBitmap( &offscrMap );

   EraseRect( &offscrPort.portRect );

   /* Read BMP File to the ofscreen bitmap */

   printf("Reading BMP file into offscreen bitmap\n");

   i = ReadBMP( argv[1], NULL, NULL );
   if( i == False )  {
      CloseBitmap( &offscrMap );
      printf("BMP file not compatible with display mode\n");
      StopGraphics();
      exit(4);
   }

   /* switch back to original port */
   SetPort( defaultPort );

   /* clear the screen */
   ScreenRect( &scrnR );
   EraseRect( &scrnR );

   /* put into graphics mode */
   SetDisplay( GrafPg0 ); 

   /* set palette, based on palette in BMP file */
   WritePalette( 0, 0, (int) QueryColors(), RGBInfo );

   /* transfer bmp image from offscreen bitmap to default bitmap */
   CopyBlit( &offscrPort, defaultPort,
             &offscrPort.portRect, &defaultPort->portRect);


   /*  wait for a key press */
   getch();

   /* free the offscreen bitmap */
   CloseBitmap( &offscrMap );


   SetDisplay( TextPg0 );
   StopGraphics();
   i = QueryError();
   printf("QueryError=%d/%d\n",i >> 7,i & 127);
   exit(i);

}

#include "metquery.c"

