/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 2    BMPREMAP.C    4-May-94    16:26:00 #    */
/* ************************************************** */
/*    METAGRAPHICS SOFTWARE CORPORATION (c) 1993      */
/*    Demonstrates use of incremental BMP operations  */ 
/*    to re-map 'reserved' colors used by a file to   */
/*    the closest other available colors              */
/*    Functions: OpenBMP(), NextBMP(), CloseBMP()     */
/*               XlateImage(), FindClosestRGB()       */
/* ************************************************** */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>

#include "metawndo.h"   /* master MetaWINDOW include file */
#include "bmplab.h"     /* BMP lab include file */


/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /* stack size 14K */
#endif

/* global vaiables */
int   GrafixCard,GrafixInput;

/* function prototype for subroutine  */
int  XlateBMP( char  *filename, long *xltable );

void main( int argc, char *argv[] )
{
   int      i;
   rect     scrnR;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   palData  RGBInfo[256];     /* buffer for reading the files palette    */

   long     xTable[256];      /* color translation table */
   palData  RsvdRGBInfo[16];  /* buffer for holding reserved RGB values  */

   /* check for a file name on the command line */
   if( argc < 2 ) {
      printf("BMPREMAP - Remap colors in a BMP file\n");
      printf("Usage: BMPREMAP filename\n");
      exit(10);
   }

   /* let user select display type */
   MetQuery(argc,argv);      

   /* init the system */
   i = InitGraphics( GrafixCard );
   if( i != 0 ) {
      printf("MetaWINDOW Initialization error #%d\n", i);
      exit(1);
   }

   /* read header information */
   i = ReadBMPInfo( argv[1], &bmpHeader, RGBInfo );
   if( i == 0) {
      printf("%s is not a valid bmp file\n", argv[1] );
      StopGraphics();
      exit(2);     /* not  a valid bmp file */
   }

   /* put into graphics mode */
   SetDisplay( GrafPg0 ); 

   /* clear the screen */
   ScreenRect( &scrnR );
   EraseRect( &scrnR );

   /* draw something in the reserved colors */
   MoveTo( 10, scrnR.Ymax - 50 );
   for( i = 0; i < 16; i++ ) {
      PenColor( i );
      DrawChar( 'A' + i );
   }
   scrnR.Ymax -= 100;
   ClipRect( &scrnR );

/* now generate a translation table to re-map those colors in the BMP
file that are in reserved locations to other locations. This sample 
assumes that the first 15 colors in the palette are 'reserved' */

   /* record the bmp palette entries that are in reserved locations */
   for( i = 0; i < 16; i++ ) 
      RsvdRGBInfo[i] = RGBInfo[i];

   /* replace the reserved palette data with the current palette settings */
   ReadPalette( 0, 0, 15, RGBInfo );

   /* set palette, based on palette in BMP file */
   WritePalette( 0, 0, (int) QueryColors(), RGBInfo );

   // start with a 1:1 translation table
   for( i = 0; i < 256; i ++ ) 
      xTable[i] = (long) i;

   /* translate the reserved colors of the bmp file to others
   in the current displays palette */
   for( i = 0; i < 16; i++ ) 
      xTable[i] = (long) FindClosestRGB( &RsvdRGBInfo[i], RGBInfo );


   /* read it in */
   i = XlateBMP( argv[1], xTable );

   /*  wait for a key press */
   getch();


   /* switch back to text mode*/
   SetDisplay( TextPg0 );

   /* display any error messages */
   switch( i ) {
      case 1: /* file read correctly */
         break;
      case -1:
         printf("%s is not a valid bmp file\n", argv[1] );
         break;
      case -2: 
         printf("can't allocate image buffer\n");
         break;
      case -3: 
         printf("can't allocate translation buffer\n");
         break;
      default: 
         printf("error reading BMP file\n");
         break;
   }

   StopGraphics();
   i = QueryError();
   printf("QueryError=%d/%d\n",i >> 7,i & 127);
   exit(i);

}


/*
XlateBMP: read and translate a bmp file to be compatible with current display
          format and translate colors per xltable   

Returns;
    1 = file read correctly
    0 = error reading BMP file
   -1 = not a valid bmp file
   -2 = can't allocate image buffer
   -3 = can't allocate translation buffer

*/

int  XlateBMP( char  *filename, long *xltable )

{
   int      i, height,y;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   unsigned imsize;           /* size of required image buffer           */
   image    *srcImage;        /* pointer to allocated source image buffer*/
   image    *dstImage;        /* pointer to allocated dest image buffer  */
   grafPort *myport;          /* pointer to current port                 */     
   rect     dstR;             /* rectangle used to WriteImage() raster   */


   /* Open the BMP File and read header information */
   imsize =  OpenBMP( filename, &bmpHeader, NULL );
   if( imsize == 0)
      return -1;     /* not  a valid bmp file */

   GetPort( &myport );

   /* file needs to be color translated */
   /* each raster will be processed individually  */ 

   /* allocate an image buffer large enough for one raster of the bmp file */
   srcImage = malloc( imsize );
   if( srcImage == NULL ) {
      CloseBMP( &bmpHeader );
      return  -2;    /*  can't allocate */
   }

   /* calculate a translation image buffer size based on source image */

   /* get a 'taste' of the bmp files image format */
   NextBMP( &bmpHeader, srcImage );
   imsize  = (unsigned) XlateImage( srcImage, NULL, myport->portMap->pixBits,
                                    myport->portMap->pixPlanes, NULL ); 
   dstImage = malloc( imsize );
   if( dstImage == NULL ) {
      free( srcImage );
      CloseBMP( &bmpHeader );
      return -3;     /*  can't allocate */
   }

   /* reset file to beginning */
   CloseBMP( &bmpHeader );
   OpenBMP( filename, &bmpHeader, NULL );

   /* set up a rectangle for the destination WriteImage() */
   ScreenRect( &dstR );
   dstR.Ymin = dstR.Ymax - 1;

   /* compute number of rasters in the bmp file */
   height = bmpHeader.height;

   /* process each bmp file raster individually */
   for( y = height; y > 0; y-- ) {

      /* read next raster into an image buffer */
      i = NextBMP( &bmpHeader, srcImage );

      if( i == False )  {
         CloseBMP( &bmpHeader );
         free( srcImage );
         free( dstImage );
         return 0;      /* error reading */
      }

      /*  translate the image to destination format and colors */
      XlateImage( srcImage, dstImage, myport->portMap->pixBits,
                  myport->portMap->pixPlanes, xltable ); 
      
      /* put the translated image on the current bitmap */
      WriteImage( &dstR, dstImage );

      /* advance destination rectangle to next raster*/
      OffsetRect( &dstR, 0, -1 );

   } /* for Y */


   /* close the BMP file opened via OpenBMP */
   CloseBMP( &bmpHeader );

   /* free the image buffers */
   free( srcImage );
   free( dstImage );

   return 1;

}

#include "metquery.c"

