/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 3    BMPSCALE.C    31-Mar-93    13:12:44 #    */
/* ************************************************** */
/*    METAGRAPHICS SOFTWARE CORPORATION (c) 1993      */
/*    Demonstrates use of incremental BMP operations  */ 
/*    to scale a file to a rectangle                  */
/*    Functions: OpenBMP(), NextBMP(), and CloseBMP() */
/* ************************************************** */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>

#include "metawndo.h"   /* master MetaWINDOW include file */
#include "bmplab.h"     /* BMP lab include file */


/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /* stack size 14K */
#endif


/* global vaiables */
int   GrafixCard,GrafixInput; 
short bits,planes;
grafPort *thePort;

/* function prototype  */
int  ScaleBMP( char  *filename, rect *dest );

void main( int argc, char *argv[] )
{
   int      i;
   rect     scrnR;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   palData  RGBInfo[256];     /* buffer for reading the files palette    */

   /* check for a file name on the command line */
   if( argc < 2 ) {
	  printf("BMPSCALE - Read and scale BMP file\n");
	  printf("Usage: BMPSCALE filename\n");
	  exit(10);
   }

   /* let user select display type */
   MetQuery(argc,argv);      

   /* init the system */
   i = InitGraphics( GrafixCard );
   if( i != 0 ) {
	  printf("MetaWINDOW Initialization error #%d\n", i);
	  exit(20);
   }

   if ( ReadBMPInfo( argv[1], &bmpHeader, RGBInfo ) == False )  {
	  printf("%s is not a valid BMP file\n", argv[1]);
	  StopGraphics();
	  exit(2);
   }

   /* put into graphics mode */
   SetDisplay( GrafPg0 ); 

   /* clear the screen */
   ScreenRect( &scrnR );
   EraseRect( &scrnR );

   /* set palette, based on palette in BMP file */
   WritePalette( 0, 0, (int) QueryColors(), RGBInfo );
   
   GetPort(&thePort);
   bits=thePort->portMap->pixBits;
   planes=thePort->portMap->pixPlanes;
   
   /* read it in normally */
   ReadBMP( argv[1], NULL, NULL );

   /* read in scaled down */
   scrnR.Ymin  = scrnR.Ymax * 3 / 4;
   scrnR.Xmin  = scrnR.Xmax * 3 / 4;

//  SetRect(&scrnR,0,0,bmpHeader.width/2,bmpHeader.height/2);   
   i = ScaleBMP( argv[1], &scrnR );
	
   /*  wait for a key press */
   getch();

   /* switch back to text mode*/
   SetDisplay( TextPg0 );

   /* display any error messages */
   switch( i ) {
	  case 1: /* file read correctly */
		 break;
	  case -1:
		 printf("%s is not a valid bmp file\n", argv[1] );
		 break;
	  case -2: 
		 printf("can't allocate image buffer\n");
		 break;
	  case -3: 
		 printf("can't allocate translation buffer\n");
		 break;
	  default: 
		 printf("error reading BMP file\n");
		 break;
   }

   StopGraphics();
   i = QueryError();
   printf("QueryError=%d/%d\n\n",i >> 7,i & 127);
   printf("bmpHeader.bits= %i, bmpHeader.planes= %i\n",
		bmpHeader.bits,bmpHeader.planes);
   printf("display.bits= %i, display.planes= %i\n",
		bits,planes);
   exit(i);

}


/*
ScaleBMP: read and scale a bmp file

Returns;
	1 = file read correctly
	0 = error reading BMP file
   -1 = not a valid bmp file
   -2 = can't allocate image buffer
   -3 = can't allocate translation buffer

*/

int  ScaleBMP( char  *filename, rect *dstR )

{
   int       i, bmpHeight, y;
   bmpHead   bmpHeader;       /* buffer for reading the files BMP header */
   unsigned  imsize;          /* size of required image buffer           */
   image     *srcImage;       /* pointer to allocated source image buffer*/
   image     *dstImage;       /* pointer to allocated dest image buffer  */
   rect      wiR;             /* rectangle used to WriteImage() raster   */
   int       ratio, ratioCntr, xsize;   
   long     l;

   /* Open the BMP File and read header information */
   imsize =  OpenBMP( filename, &bmpHeader, NULL );
   if( imsize == 0)
	  return -1;     /* not  a valid bmp file */

   /* allocate an image buffer large enough for one raster of the bmp file */
   srcImage = malloc( imsize );
   if( srcImage == NULL ) {
	  CloseBMP( &bmpHeader );
	  return  -2;    /*  can't allocate */
   }

   /* calculate a translation image buffer size */

   /* get a  'taste' of the bmp files image format */
   NextBMP( &bmpHeader, srcImage );
   xsize = dstR->Xmax - dstR->Xmin;
   imsize  = (unsigned) ScaleImage( srcImage, NULL, xsize, 1  );
   dstImage = malloc( imsize );
   if( dstImage == NULL ) {
	  free( srcImage );
	  CloseBMP( &bmpHeader );
	  return -3;     /*  can't allocate */
   }

   /* reset file to beginning */
   CloseBMP( &bmpHeader );
   OpenBMP( filename, &bmpHeader, NULL );

   /* set up a rectangle for the destination WriteImage() */
   DupRect( dstR, &wiR );
   wiR.Ymin = wiR.Ymax - 1;

   /* compute number of rasters in the bmp file */
   bmpHeight = bmpHeader.height;

   /* compute fixed point ratio of dest rect size to bmp file size */
   l = (long) (dstR->Ymax - dstR->Ymin);
   l = l * 100L + (long) (bmpHeight - 1 );
   ratio  = (int) ( l / (long) bmpHeight );

   /* start off running total at 0 */
   ratioCntr = 0;

   /* process each bmp file raster individually */
   for( y = bmpHeight; y >0; y-- ) {


	  /* read next raster into an image buffer */
	  i = NextBMP( &bmpHeader, srcImage );

	  if( i == False )  {
		 CloseBMP( &bmpHeader );
		 free( srcImage );
		 free( dstImage );
		 return 0;      /* error reading */
	  }

	  /* advance by ratio amount */
	  ratioCntr += ratio;

	  /* as long as we have a whole part of a ratio count */
	  while( ratioCntr / 100 ) {

		 ratioCntr -= 100;    // do 1 whole unit,leave fractional part

		 if( wiR.Ymin <= dstR->Ymin )
			break;

		 /* scale raster line */
		 ScaleImage( srcImage, dstImage, xsize, 1 ); 

		 /* write this raster line */
		 WriteImage( &wiR, dstImage );
		 OffsetRect( &wiR, 0, -1 );
	  }


   } /* for Y */


   /* close the BMP file opened via OpenBMP */
   CloseBMP( &bmpHeader );

   /* free the image buffers */
   free( srcImage );
   free( dstImage );

   return 1;

}

#include "metquery.c"

