/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 4    BMPXLATE.C    31-Mar-93    13:14:06 #    */
/* ************************************************** */
/*    METAGRAPHICS SOFTWARE CORPORATION (c) 1993      */
/*    Demonstrates use of incremental BMP operations  */ 
/*    to translate a file to the current display type */
/*    Functions: OpenBMP(), NextBMP(), and CloseBMP() */
/* ************************************************** */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>

#include "metawndo.h"   /* master MetaWINDOW include file */
#include "bmplab.h"     /* BMP lab include file */


/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /* stack size 14K */
#endif


/* global vaiables */
int   GrafixCard,GrafixInput;

/* function prototype  */
int  XlateBMP( char  *filename );

void main( int argc, char *argv[] )
{
   int      i;
   rect     scrnR;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   palData  RGBInfo[256];     /* buffer for reading the files palette    */

   /* check for a file name on the command line */
   if( argc < 2 ) {
      printf("BMPXLATE - Read and translate BMP file\n");
      printf("Usage: BMPXLATE filename\n");
      exit(10);
   }

   /* let user select display type */
   MetQuery(argc,argv);      

   /* init the system */
   i = InitGraphics( GrafixCard );
   if( i != 0 ) {
      printf("MetaWINDOW Initialization error #%d\n", i);
      exit(1);
   }

   /* read header information */
   i = ReadBMPInfo( argv[1], &bmpHeader, RGBInfo );
   if( i == 0) {
      printf("%s is not a valid bmp file\n", argv[1] );
      StopGraphics();
      exit(2);     /* not  a valid bmp file */
   }

   /* put into graphics mode */
   SetDisplay( GrafPg0 ); 

   /* clear the screen */
   ScreenRect( &scrnR );
   EraseRect( &scrnR );

   /* set palette, based on palette in BMP file */
   WritePalette( 0, 0, (int) QueryColors(), RGBInfo );

   /* read  it in */
   i = XlateBMP( argv[1] );

   /*  wait for a key press */
   getch();

   /* switch back to text mode*/
   SetDisplay( TextPg0 );

   /* display any error messages */
   switch( i ) {
      case 1: /* file read correctly */
         break;
      case -1:
         printf("%s is not a valid bmp file\n", argv[1] );
         break;
      case -2: 
         printf("can't allocate image buffer\n");
         break;
      case -3: 
         printf("can't allocate translation buffer\n");
         break;
      default: 
         printf("error reading BMP file\n");
         break;
   }

   StopGraphics();
   i = QueryError();
   printf("QueryError=%d/%d\n",i >> 7,i & 127);
   exit(i);

}


/*
XlateBMP: read and translate a bmp file to be compatible with current display 

Returns;
    1 = file read correctly
    0 = error reading BMP file
   -1 = not a valid bmp file
   -2 = can't allocate image buffer
   -3 = can't allocate translation buffer

*/

int  XlateBMP( char  *filename )

{
   int      i, height,y;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   unsigned imsize;           /* size of required image buffer           */
   image    *srcImage;        /* pointer to allocated source image buffer*/
   image    *dstImage;        /* pointer to allocated dest image buffer  */
   grafPort *myport;          /* pointer to current port                 */     
   rect     dstR;             /* rectangle used to WriteImage() raster   */


   /* Open the BMP File and read header information */
   imsize =  OpenBMP( filename, &bmpHeader, NULL );
   if( imsize == 0)
      return -1;     /* not  a valid bmp file */

   /* check if this file is compatible with current video mode */ 
   GetPort( &myport );
   if( bmpHeader.bits == myport->portMap->pixBits && 
       bmpHeader.planes == myport->portMap->pixPlanes ) {

      /* it's compatible, do it the easy way */
      CloseBMP( &bmpHeader );
      return ReadBMP( filename, NULL, NULL );
   }


   /* file needs to be translated to current video format */
   /* each raster will be processed individually          */ 

   /* allocate an image buffer large enough for one raster of the bmp file */
   srcImage = malloc( imsize );
   if( srcImage == NULL ) {
      CloseBMP( &bmpHeader );
      return  -2;    /*  can't allocate */
   }

   /* calculate a translation image buffer size based on source image */

   /* get a 'taste' of the bmp files image format */
   NextBMP( &bmpHeader, srcImage );
   imsize  = (unsigned) XlateImage( srcImage, NULL, myport->portMap->pixBits,
                                    myport->portMap->pixPlanes, NULL ); 
   dstImage = malloc( imsize );
   if( dstImage == NULL ) {
      free( srcImage );
      CloseBMP( &bmpHeader );
      return -3;     /*  can't allocate */
   }

   /* reset file to beginning */
   CloseBMP( &bmpHeader );
   OpenBMP( filename, &bmpHeader, NULL );

   /* set up a rectangle for the destination WriteImage() */
   ScreenRect( &dstR );
   dstR.Ymin = dstR.Ymax - 1;    // START AT BOTTOM  for bmp files

   /* compute number of rasters in the bmp file */
   height = bmpHeader.height;

   /* process each bmp file raster individually */
   for( y = height; y > 0; y-- ) {

      /* read next raster into an image buffer */
      i = NextBMP( &bmpHeader, srcImage );

      if( i == False )  {
         CloseBMP( &bmpHeader );
         free( srcImage );
         free( dstImage );
         return 0;      /* error reading */
      }

      /*  translate the image to destination format */
      XlateImage( srcImage, dstImage, myport->portMap->pixBits,
                  myport->portMap->pixPlanes, NULL ); 
      
      /* put the translated image on the current bitmap */
      WriteImage( &dstR, dstImage );

      /* advance destination rectangle to next raster*/
      OffsetRect( &dstR, 0, -1 );

   } /* for Y */


   /* close the BMP file opened via OpenBMP */
   CloseBMP( &bmpHeader );

   /* free the image buffers */
   free( srcImage );
   free( dstImage );

   return 1;

}

#include "metquery.c"

