/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 1    BMPXLRGB.C    9-May-94    11:08:54 #    */
/* ************************************************** */
/*   METAGRAPHICS SOFTWARE CORPORATION (c) 1993       */
/*   Demonstrates use of incremental BMP operations   */ 
/*   to translate a file to the current display type  */
/*   and convert colors to/from RGB/palette indexes   */
/*   Functions: OpenBMP(), NextBMP(), and CloseBMP()  */
/*              FindClosestRGB()                      */
/* ************************************************** */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>

#include "metawndo.h"   /* master MetaWINDOW include file */
#include "bmplab.h"     /* BMP lab include file */


/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /*stack size 14K */
#endif

/* global vaiables */
int   GrafixCard,GrafixInput;

/* function prototype  */
int  XlateBMP( char  *filename, long *xltable );
void TranslateTable( long *xlt, bmpHead *bmphdr, palData *palette );

void main( int argc, char *argv[] )
{
   int      i;
   rect     scrnR;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   palData  RGBInfo[256];     /* buffer for reading the files palette    */
   long     xTable[512];      /* color translation table */

   /* check for a file name on the command line */
   if( argc < 2 ) {
      printf("BMPXLRGB - Read and translate BMP file\n");
      printf("Usage: BMPXLRGB filename\n");
      exit(10);
   }

   /* let user select display type */
   MetQuery(argc,argv);      

   /* init the system */
   i = InitGraphics( GrafixCard );
   if( i != 0 ) {
      printf("MetaWINDOW Initialization error #%d\n", i);
      exit(1);
   }

   /* read header information */
   i = ReadBMPInfo( argv[1], &bmpHeader, RGBInfo );
   if( i == 0) {
      printf("%s is not a valid bmp file\n", argv[1] );
      StopGraphics();
      exit(2);     /* not  a valid bmp file */
   }

   /* put into graphics mode */
   SetDisplay( GrafPg0 ); 

   /* clear the screen */
   ScreenRect( &scrnR );
   EraseRect( &scrnR );

   /* set palette, based on palette in BMP file */
   WritePalette( 0, 0, (int) QueryColors(), RGBInfo );

   /* build a color value translation table */
   TranslateTable( xTable, &bmpHeader, RGBInfo );

   /* read  it in */
   i = XlateBMP( argv[1], xTable );

   /*  wait for a key press */
   getch();

   /* switch back to text mode*/
   SetDisplay( TextPg0 );

   /* display any error messages */
   switch( i ) {
      case 1: /* file read correctly */
         break;
      case -1:
         printf("%s is not a valid bmp file\n", argv[1] );
         break;
      case -2: 
         printf("can't allocate image buffer\n");
         break;
      case -3: 
         printf("can't allocate translation buffer\n");
         break;
      default: 
         printf("error reading BMP file\n");
         break;
   }

   StopGraphics();
   i = QueryError();
   printf("QueryError=%d/%d\n",i >> 7,i & 127);
   exit(i);

}


/*
XlateBMP: read and translate a bmp file to be compatible with current display 
          format, translate colors as specified by xltable
Returns;
    1 = file read correctly
    0 = error reading BMP file
   -1 = not a valid bmp file
   -2 = can't allocate image buffer
   -3 = can't allocate translation buffer

*/

int  XlateBMP( char  *filename, long *xltable )

{
   int      i, height,y;
   bmpHead  bmpHeader;        /* buffer for reading the files BMP header */
   unsigned imsize;           /* size of required image buffer           */
   image    *srcImage;        /* pointer to allocated source image buffer*/
   image    *dstImage;        /* pointer to allocated dest image buffer  */
   grafPort *myport;          /* pointer to current port                 */     
   rect     dstR;             /* rectangle used to WriteImage() raster   */


   /* Open the BMP File and read header information */
   imsize =  OpenBMP( filename, &bmpHeader, NULL );
   if( imsize == 0)
      return -1;     /* not  a valid bmp file */

   /* check if this file is compatible with current video mode */ 
   GetPort( &myport );
   if( bmpHeader.dspColors == bmpHeader.bmpColors){ 

      /* it's compatible, do it the easy way */
      CloseBMP( &bmpHeader );
      return ReadBMP( filename, NULL, NULL );
   }


   /* file needs to be translated to current video format */
   /* each raster will be processed individually          */ 

   /* allocate an image buffer large enough for one raster of the bmp file */
   srcImage = malloc( imsize );
   if( srcImage == NULL ) {
      CloseBMP( &bmpHeader );
      return  -2;    /*  can't allocate */
   }

   /* calculate a translation image buffer size based on source image */

   /* get a 'taste' of the bmp files image format */
   NextBMP( &bmpHeader, srcImage );
   imsize  = (unsigned) XlateImage( srcImage, NULL, myport->portMap->pixBits,
                                    myport->portMap->pixPlanes, NULL ); 
   dstImage = malloc( imsize );
   if( dstImage == NULL ) {
      free( srcImage );
      CloseBMP( &bmpHeader );
      return -3;     /*  can't allocate */
   }

   /* reset file to beginning */
   CloseBMP( &bmpHeader );
   OpenBMP( filename, &bmpHeader, NULL );


   /* set up a rectangle for the destination WriteImage() */
   ScreenRect( &dstR );
   dstR.Ymin = dstR.Ymax - 1;

   /* compute number of rasters in the bmp file */
   height = bmpHeader.height;

   /* process each bmp file raster individually */
   for( y = height; y > 0; y-- ) {

      /* read next raster into an image buffer */
      i = NextBMP( &bmpHeader, srcImage );

      if( i == False )  {
         CloseBMP( &bmpHeader );
         free( srcImage );
         free( dstImage );
         return 0;      /* error reading */
      }

      /*  translate the image to destination format */
      XlateImage( srcImage, dstImage, myport->portMap->pixBits,
                  myport->portMap->pixPlanes, xltable ); 
      
      /* put the translated image on the current bitmap */
      WriteImage( &dstR, dstImage );

      /* advance destination rectangle to next raster*/
      OffsetRect( &dstR, 0, -1 );

   } /* for Y */


   /* close the BMP file opened via OpenBMP */
   CloseBMP( &bmpHeader );

   /* free the image buffers */
   free( srcImage );
   free( dstImage );

   return 1;

}

/* build a color translation table */
void TranslateTable( long *xlt, bmpHead *bmphdr, palData *palette )
{

   grafPort *thePort;      /* current port */
   long  dstcolors;        /* number of colors in port */
   long  srcolors;         /* number of colors in bmp file */
   palData  RGB;
   int i;

   /* how many colors in source bmp file ? */
   srcolors = 1;
   srcolors = srcolors << (bmphdr->bits * bmphdr->planes);
   srcolors--;

   /* how many colors in current port ? */
   dstcolors = QueryColors();

   /* going from palette based to RGB ? */
   if( srcolors <= 255 && dstcolors > 255 ) {
      /*
      Make a translate table that will convert palette references into
      16 bit RGB values
      */
      GetPort( &thePort );
      for( i = 0; i < 256; i++ ) {
         if( thePort->portMap->mapFlags & mf565 ) {
            // 5:6:5 RGB format
            xlt[i]  = (palette[i].palRed         ) & 0xF800;
            xlt[i] |= (palette[i].palGreen >> 5  ) & 0x07E0;
         }
         else {
            // 5:5:5 RGB format
            xlt[i]  = ( palette[i].palRed  >> 1  ) & 0x7C00;
            xlt[i] |= (palette[i].palGreen >> 6  ) & 0x03E0;
         }
         xlt[i] |= (palette[i].palBlue  >> 11 ) & 0x001F;
      }

      return;
   }

   /* going from RGB based to palette based ? */
   if( srcolors > 255 && dstcolors <= 255 ) {
      /*
      Make a translate table that will reduce a 24 bit per pixel source
      to colors in the current (hopefull optimized for this image) palette.
      XlateImage reduces the colors to the 512 most significant,
      then we translate those to the best 256 we got
      */
      for( i = 0; i < 512; i++ ) {
         RGB.palRed   = (i & 0x1C0) << 7;    //  3 msbs of red
         RGB.palGreen = (i & 0x038) << 10;   //  3 msbs of green
         RGB.palBlue  = (i & 0x007) << 13;   //  3 msbs of blue
         xlt[i] = (long) FindClosestRGB( &RGB, palette );
      }

      return;
   }

   /* make a null translate table (no color conversion) 
   when the source color values are compatible with the current display */
   for( i = 0; i < 256; i++ )
      xlt[i] = (long) i;

}

#include "metquery.c"

