/**********************************************************************/
/*                                                                    */
/*    Program: McShell     Module: bmplab.c                           */
/*                                                                    */
/*--------------------------------------------------------------------*/
/*                                                                    */
/*    Description    : Definitions for BMP format and functions       */
/*                     that manipulate it                             */
/*--------------------------------------------------------------------*/
/*                                                                    */
/*    Author         : MICHAEL D. MANNING                             */
/*    developed on   : 01/17/1996                                     */    
/*    last update    : 02/26/1996                                     */
/*                                                                    */
/*  	COMPILER OPTIONS: MSVC++ 1.5                                    */ 
/*		/nologo /Gs /G2 /W2 /AL /Gx- /D "NDEBUG" /D "_DOS" /FR          */
/*                                                                    */
/*--------------------------------------------------------------------*/
/*                                                                    */
/*    COPYRIGHT (C) 1996, MIKE MANNING, CCI SOFTWARE                  */
/*                                                                    */
/**********************************************************************/

#include <stdio.h> 
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <bios.h>
#include <time.h>
#include <MetaWndo.h>
#include "bmplab.h"

void InitBMPFileHeader(BMPFILEHEADER *bmpFileHeader );
void InitRGBTriple(RGBTRIPLE *rgbTriple );
void SetRGBTriple( int r, int g, int b, RGBTRIPLE *rgbTriple );
void InitBmpCoreHeader(BMPCOREHEADER * bmpCoreHdr );
void InitRGBQuad(RGBQUAD *rgbQuad );
void SetRGBQuad( int r, int g, int b, RGBQUAD *rgbQuad );
void InitBMPInfoHeader(BMPINFOHEADER *bmpInfoHdr );
int  GetBMPInfo(void *BUF, BMPHEADER *bmpHdr, palData *imgColors);
void BmpBUFRead_none( unsigned char *buf, int npxls );
void BmpBUFRead_rle4( unsigned char *buf, int npxls );
void BmpBUFRead_rle8( unsigned char *buf, int npxls );
int iscale(int i, int n, int d);
void far DrawStringFXY(int x, int y, char *ps, ...);
void BuildScaleTable(float size,int tsize);
void BuildScaleTable2(float size, int width);
unsigned char ExtractBit( int bit, char buf[]);


palData   BMPColors[256];
int bmp_width,bmp_height;
int meta_hd_size; 

int cond;
int rle_method;
int pxl_size;
FILE *bmp_file;
char *bmp_buf;

unsigned char BitTable[]={128,64,32,16,8,4,2,1};
unsigned int ScaleTable[1281];


long SEQ  xScaleImage(image *SRCIMAGE, image *DSTIMAGE,
                     int DSTWIDTH, int DSTHEIGHT )
{
	register int h,i,j,k;
	int bit;
	static float aspect;
	imageHeader *sImg,*dImg;
	image *srcBuf,*dstBuf;
	image *srcBase,*dstBase;
	rect r;
	
	if(!SRCIMAGE) return 0;
	if(!DSTIMAGE) return (long) ((DSTHEIGHT*(DSTWIDTH+1))*5);

	sImg=(imageHeader *)SRCIMAGE;
	dImg=(imageHeader *)DSTIMAGE;
	
  	
	srcBase=SRCIMAGE+12;
	dstBase=DSTIMAGE+12;
	if(ScaleTable[0]!=DSTWIDTH){
		ScaleTable[0]=DSTWIDTH;
   	aspect=(float)sImg->imWidth/(float)DSTWIDTH; 
		BuildScaleTable(aspect,DSTWIDTH);
	   SetRect(&r,0,0,DSTWIDTH,1);
		ReadImage(&r,DSTIMAGE); 
	}


	switch(sImg->imBits){
		case 1:		
			for(h=0; h<sImg->imPlanes; h++){
				for(i=0; i<DSTHEIGHT; i++){
					dstBuf=dstBase;
					srcBuf=srcBase;
					bit=1;   //skip first byte, ScaleTable[0] = DSTWIDTH
					for (j=0; j<dImg->imRowBytes; j++){  // get byte loop
						*dstBuf=0;
						for(k=0; k<8; k++){  // get pixel loop
								if(ExtractBit(ScaleTable[bit++],srcBuf))
								*dstBuf += BitTable[k];
						}
						dstBuf++;
					}
					dstBase+=dImg->imRowBytes;    // inc dst to next plane
					srcBase+=sImg->imRowBytes;    // inc src to next plane
				}
				//dstBase+=dImg->imRowBytes;    // inc dst to next plane
				//srcBase+=sImg->imRowBytes;    // inc src to next plane
			}
			break;
		default:     
			for(h=0; h<DSTHEIGHT; h++){
				dstBuf=dstBase;
				srcBuf=srcBase+ScaleTable[h+1]*sImg->imWidth;
				for (i=0; i<DSTWIDTH; i++) *dstBuf++ = *(srcBuf+ScaleTable[i+1]);
				dstBase+=dImg->imWidth; 
			}
		break;
	}

	return 0;
	
}

void BuildScaleTable(float ratio, int width)
{
	register int i;

/*
	PenColor(0);
	BackColor(White);
	DrawStringFXY(10,460,"ratio=%f, tsize=%i               ",ratio,tsize); 
*/ 
	for (i=0; i<=width; i++)
		ScaleTable[i+1] = (int)((float)i*ratio);  // Shift out fraction.
}

unsigned char ExtractBit( int bit, char buf[])
{
	int i,r;
	unsigned char *p;
	
	p=buf;
	
	if(bit<8) return (*p & BitTable[bit]) ? 1 : 0 ;   // in 1st. byte of buf
		
	i=bit/8;			// get byte in buf
	r=bit%8;       // get offset into byte

	// point to correct byte in buf
	return (*(p+i) & BitTable[r]) ? 1 : 0 ;  // return bit
} 
	
//.......Save the BMP file
int SEQ WriteBMP(char *FNAME, rect *SOURCER, palData *RGBColors)
{
	int i,j;
	int nrows,ncols,nbits,ncolors,rowbytes,maxcolors; 
	long bmofs,fisiz;
	FILE *f;
	RGBTRIPLE pal[256];
	image *pkPtr,*scanln,*buf;
	rect sR;
	grafPort *wPort;
   BMPFILEHEADER fh;
   BMPINFOHEADER ih;
   RGBQUAD x;

	GetPort(&wPort);
	if( SOURCER )
		DupRect( SOURCER, &sR );
	else DupRect(&wPort->portRect,&sR);
	meta_hd_size=(sizeof(imageHeader));
	
	nrows   = sR.Ymax-sR.Ymin;
	ncols   = sR.Xmax-sR.Xmin;
	//int ncolors = QueryColors() +1;
	nbits=wPort->portMap->pixBits*wPort->portMap->pixPlanes;
	ncolors  = (nbits < 24) ? 1 << nbits : 0;
	rowbytes = ncols*nbits;
	maxcolors=QueryColors()+1;

   //use 5*rowbytes for XlateImage if image is 4 planes

   scanln = (image *)malloc((unsigned int)rowbytes*5);
   if( scanln == NULL ) return 0;
   
   buf = (image *)malloc((unsigned int)rowbytes*5);

   if( buf == NULL ) {
		free(scanln);
		return  0;
   }


   f = fopen( FNAME, "wb" );
   if( f == 0 ){
		free(scanln);
		free(buf);
		return  0;
   }


   // write an empty file header as a place holder
		InitBMPFileHeader(&fh);   
		if( fwrite( &fh, sizeof(BMPFILEHEADER), 1, f ) != 1 ){
		fclose(f);
		free(scanln);
		free(buf);
		return  0;
   }


   // initialize and write the image header
	InitBMPInfoHeader(&ih);
   ih.biWidth = ncols;
   ih.biHeight = nrows;
   ih.biPlanes = 1;
   ih.biBitCount = nbits;
   ih.biSizeImage = ((long) nrows)  * ((long) rowbytes);
   if( fwrite( &ih, sizeof(BMPINFOHEADER), 1, f ) != 1 ){
		fclose(f);
		free(scanln);
		free(buf);
		return  0;
	}


   // write the palette

	
	for( i=0; i<ncolors; i++ ){

		RGBColors[i].palRed /=256;
		RGBColors[i].palGreen /= 256;
		RGBColors[i].palBlue /= 256;
		pal[i].red=(unsigned char)RGBColors[i].palRed;
		pal[i].green=(unsigned char)RGBColors[i].palGreen;
		pal[i].blue=(unsigned char)RGBColors[i].palBlue;

		SetRGBQuad( pal[i].red, pal[i].green, pal[i].blue,&x );
		if( fwrite( &x, sizeof(RGBQUAD), 1, f ) != 1 ){
			fclose(f);
			free(scanln);
			free(buf);
			return 0;
	   }
	}


   // save offset to start of bitmap
   bmofs = ftell( f );


   // output the bitmap
   BmpEncoder( f, nbits );

	sR.Ymin =sR.Ymax- 1;   //bmp file is upside down


   if(maxcolors>16){       //256 color mode
	   for( i=nrows-1; i>=0; i-- ){
			ReadImage(&sR,scanln);
			sR.Ymin -= 1;
			sR.Ymax -= 1;
	
//			for(j=0;j<ncols;j++)
//				buf[j]=scanln[j+meta_hd_size];  //minus imageHeader (12 bytes) for ReadImage buf record
	
			BmpEncode( scanln+meta_hd_size, ncols);
			if( BmpStatus() != xOKAY ){
				free(scanln);
				free(buf);
				fclose( f );
			return 0;
			}
		}
	} //end of if maxcolors >16

	if(maxcolors<=16){      // 16 color mode
		if( ( pkPtr = (image *)( malloc( (unsigned int) rowbytes*5 ) ) )  == NULL ){
			fclose( f );
			free(scanln);
			free(buf);
			return 0 ;
		}
		for( i=nrows-1; i>=0; i-- ){
			ReadImage(&sR, pkPtr);
			sR.Ymin -= 1;
			sR.Ymax -= 1;
	
			XlateImage(pkPtr,buf,8,1,NULL);   //all images are stored in 8 bit images
			for(j=0;j<ncols;j++)
				scanln[j]=buf[j+meta_hd_size];  //add imageHeader (12 bytes) for ReadImage buf record

			BmpEncode( scanln, ncols);
			if( BmpStatus() != xOKAY ){
				free(pkPtr);
				free(scanln);
				free(buf);
				fclose( f );
				return 0;
			}
     }
	} //end of 16 color mode

   if(maxcolors<=16)free(pkPtr);
   free(scanln);
   free(buf);

   // save offset to end of file
   fisiz = ftell( f );

   // update and rewrite file header
   fh.bfSize = fisiz;
   fh.bfOffBits = bmofs;
   fseek( f, 0L, SEEK_SET );
   if( fwrite( &fh, sizeof(BMPFILEHEADER), 1, f ) != 1 ){
		return  0;
   }

   fclose( f );
   return 1;
}

int SEQ ReadBMPInfo(char *fileName, BMPHEADER *bmpHdr, palData *imgColors)
{
	register int i;
   FILE *fbmp;
	BMPINFOHEADER bmih;
	BMPCOREHEADER bmch;
   BMPFILEHEADER bmfh;
	RGBTRIPLE t;
	RGBQUAD q;
   long offset,nbytes;

   // open file
   fbmp = fopen( fileName, "rb" );
   if( fbmp == 0 )
		return 0;

   // read header and check
   if( fread(&bmfh,sizeof(BMPFILEHEADER),1,fbmp) != 1 ){
		fclose(fbmp);
		return 0;
   }

   if( bmfh.bfType != BMP_SIGNATURE_WORD ){
		fclose(fbmp);
		return 0;
   }
	bmpHdr->dspColors=QueryColors()+1;
   bmpHdr->sigWord=bmfh.bfType;
   bmpHdr->imgOffset=bmfh.bfOffBits;
   bmpHdr->bmpMWIObuf=NULL;

   // determine which format version is used
   // from structure size field
   offset = ftell( fbmp );
   nbytes = 0;
   fread( &nbytes, sizeof(long), 1, fbmp );
   fseek( fbmp, offset, SEEK_SET );

   // read bitmap header and palette

   // pre Windows 3.0 version
   if( nbytes == sizeof(BMPCOREHEADER) ){
		//DebugStr(1,10,MC_YRes-20,"pre Windows 3.0 version              ");
		if( fread(&bmch,sizeof(BMPCOREHEADER),1,fbmp) != 1 ){
			fclose(fbmp);
			return 0;
		}
		bmpHdr->width   = bmch.bcWidth;
		bmpHdr->height  = bmch.bcHeight;
		bmpHdr->bits	 = bmch.bcBitCount;
		bmpHdr->planes  = bmch.bcPlanes;
		bmpHdr->rle     = 0;
		bmpHdr->bmpColors  = 1 << bmpHdr->bmpColors;
      bmpHdr->xRes	 = 0;
      bmpHdr->yRes	 = 0;
      
		if( bmpHdr->bmpColors > 0 ){
			for(  i=0; i<bmpHdr->bmpColors; i++ ){
				if( fread(&t,sizeof(RGBTRIPLE),1,fbmp) != 1 ){
					fclose(fbmp);
					return 0;
				}
				imgColors[i].palRed    = t.red;
				imgColors[i].palGreen  = t.green;
				imgColors[i].palBlue   = t.blue;
				imgColors[i].palRed   *= 256;
				imgColors[i].palGreen *= 256;
				imgColors[i].palBlue  *= 256;
			}
		}
	}

   // Windows 3.0 and later version
   else if( nbytes == sizeof(BMPINFOHEADER) ){
	//DebugStr(1,10,MC_YRes-20,"Windows 3.0 and later version              ");
		if( fread(&bmih,sizeof(BMPINFOHEADER),1,fbmp) != 1 ){
			fclose(fbmp);
			return 0;
		}

		bmpHdr->width   = (int) bmih.biWidth;
		bmpHdr->height  = (int) bmih.biHeight;
		bmpHdr->bits    = bmih.biBitCount; 
		bmpHdr->planes  = bmih.biPlanes;
		bmpHdr->rle     = (int) bmih.biCompression;
		bmpHdr->bmpColors  = 2;
		bmpHdr->imgSize = bmih.biSizeImage;
		bmpHdr->xRes	 = bmih.biXPelsPerMeter;
		bmpHdr->yRes	 = bmih.biYPelsPerMeter; 
		bmpHdr->rowBytes= bmpHdr->bits*bmpHdr->width;
		
		switch(bmpHdr->bits){
			case 1: 
				bmpHdr->bmpColors=2;  
				break;
			case 4: 
				bmpHdr->bmpColors=16;
				break;
			case 8: 
				bmpHdr->bmpColors=256; 
				break;
		}
		if(imgColors){
			if( bmpHdr->bmpColors > 0 ){
				for( i=0; i<bmpHdr->bmpColors; i++ ){
					if( fread(&q,sizeof(RGBQUAD),1,fbmp) != 1 ) {
						fclose(fbmp);
						return 0;
					}
					imgColors[i].palRed    = q.red;
					imgColors[i].palGreen  = q.green;
					imgColors[i].palBlue   = q.blue;
					imgColors[i].palRed   *= 256;
					imgColors[i].palGreen *= 256;
					imgColors[i].palBlue  *= 256; 
				} //end of for loop
			}//end of ncolors>0 
		}//end of imgColors
	}//end of else if

   // unknown format
	else{
		fclose(fbmp);
		return 0;
	}
	fclose(fbmp);
	return 1;
}

unsigned SEQ OpenBMP(char *FNAME, BMPHEADER *bmpHeader, palData *RGBCOLORS)
{
	FILE *fbmp;
	
	if(!ReadBMPInfo(FNAME,bmpHeader,RGBCOLORS)) return 0;    
	fbmp=fopen(FNAME,"rb");
	if(fbmp==NULL) return 0;

	bmpHeader->bmpMWIObuf = (image *)malloc((unsigned int)bmpHeader->width*5);
	if( bmpHeader->bmpMWIObuf == NULL ) {
		fclose(fbmp);
		return 0;
	}
	/* move file pointer to start of aimge data */
	fseek( fbmp, bmpHeader->imgOffset, SEEK_SET );
	/* init data for Decoder */
	BmpDecoder( fbmp, bmpHeader->rle, bmpHeader->bits ); 
	BmpInit();
	/* return imgSize big enough for 24bit */
   return (unsigned int)bmpHeader->bits*bmpHeader->width*5;
}		

void SEQ CloseBMP(BMPHEADER *bmpHeader )
{
	if(bmpHeader->bmpMWIObuf)
		free(bmpHeader->bmpMWIObuf);
	BmpTerm();
	fclose(bmp_file);
}	
		
int SEQ NextBMP(BMPHEADER *bmpHeader, image *scanln)
{

	imageHeader    *imPtr;

	if(bmpHeader->bmpColors <= 16 && bmpHeader->dspColors <=16){  
	   imPtr = (imageHeader *)bmpHeader->bmpMWIObuf;  // pointer to MW image header
		imPtr->imWidth    = (int)bmpHeader->width;
	   imPtr->imHeight   = 1;
	   imPtr->imAlign    = 0;
	   imPtr->imBits     = 8;
	   imPtr->imPlanes   = 1;
	   imPtr->imRowBytes = bmpHeader->rowBytes; //can be 24 bits per pixel.. 3 planes? 
		BmpDecode( bmpHeader->bmpMWIObuf+12, bmpHeader->width );
		XlateImage(bmpHeader->bmpMWIObuf,scanln,1,4,NULL);
   }
   else{
		imPtr = (imageHeader *)scanln;  // pointer to MW image header
		imPtr->imWidth    = (int)bmpHeader->width;
	   imPtr->imHeight   = 1;
	   imPtr->imAlign    = 0;
	   imPtr->imBits     = 8;
	   imPtr->imPlanes   = 1;
	   imPtr->imRowBytes = bmpHeader->rowBytes; //can be 24 bits per pixel.. 3 planes? 
		BmpDecode( scanln+12, bmpHeader->width );
	}	
   return (BmpStatus()==0) ? 1 : 0;
}

int SEQ ReadBMP(char *FNAME, rect *SOURCER, rect *DESTR)
{
	register int i;
   BMPHEADER bmpHeader;
   palData RGBCOLORS[256];
	imageHeader    *imPtr;
	image *buf2,*pkPtr,*buf,*scanln;
	int colortable=0,err,maxcolors;
	rect sR, dR;
	int rwbytes;
	grafPort *imagePort;
   FILE *fbmp;
   long offset,nbytes;
   
	GetPort(&imagePort);
	if(!ReadBMPInfo(FNAME,&bmpHeader,RGBCOLORS)) return 0;    
   
   if(bmpHeader.bmpColors !=2)
		if(bmpHeader.dspColors != bmpHeader.bmpColors) return 0;

   if(SOURCER)
   	DupRect(SOURCER,&sR);
   else SetRect(&sR,0,0,(int)bmpHeader.width,1);
	
   sR.Ymax =sR.Ymin+bmpHeader.height;
   sR.Ymin =sR.Ymax-1;


   if(DESTR)
   	DupRect(DESTR,&dR);
   else DupRect(&imagePort->portRect,&dR);

   dR.Ymax =dR.Ymin+bmpHeader.height;
   dR.Ymin =dR.Ymax-1;
   	
	meta_hd_size=(sizeof(imageHeader));

   // open file
   fbmp = fopen( FNAME, "rb" );
   if( fbmp == 0 )
		return 0;
		
   // position to start of bitmap
   fseek( fbmp, bmpHeader.imgOffset, SEEK_SET );

   buf2 = (image *)malloc((unsigned int)bmpHeader.width*5);
	if( buf2 == NULL ) {
		fclose(fbmp);
		return 0;
	}

   if( ( pkPtr = (Byte *)( malloc( (unsigned int) rwbytes ) ) )  == NULL ){    
	   free(pkPtr); 
		fclose(fbmp);
		free(buf2);
		return -3;
	}

	
   BmpDecoder( fbmp, bmpHeader.rle, bmpHeader.bits );
   BmpInit();
   
   //ReadImage(&sR,buf);
   imPtr = (imageHeader *)pkPtr;       // pointer to MW image header
   imPtr->imWidth    = bmpHeader.width;
   imPtr->imHeight   = 1;
   imPtr->imAlign    = 0;
   imPtr->imBits     = 8;
   imPtr->imPlanes   = 1;
   imPtr->imRowBytes = bmpHeader.rowBytes; //can be 24 bits per pixel.. 3 planes? 

	if(bmpHeader.dspColors>16){
		if(bmpHeader.bmpColors==256){
		   for( i=bmpHeader.height; i>0; i-- ){
				BmpDecode( pkPtr+12, bmpHeader.width );
				WriteImage(&sR,pkPtr);
				sR.Ymin -=1;
				sR.Ymax -=1;
			}
		}
		else{
		   for( i=bmpHeader.height; i>0; i-- ){
				BmpDecode( pkPtr+12, bmpHeader.width );
				XlateImage(pkPtr,buf2,8,1,NULL);
				WriteImage(&sR,buf2);
				sR.Ymin -=1;
				sR.Ymax -=1;
			}
		}
	}
	else{
	   for( i=bmpHeader.height; i>0; i-- ){
			BmpDecode( pkPtr+12, bmpHeader.width );
			XlateImage(pkPtr,buf2,1,4,NULL);
			WriteImage(&dR,buf2);
			dR.Ymin -=1;
			dR.Ymax -=1;
		}
	}
	BmpTerm();
	fclose( fbmp );

	free(buf2);
	free(pkPtr);
   return 1;
}

int GetBMPInfo(void *BUF, BMPHEADER *bmpHdr, palData *imgColors)
{
	register int i;
   char *fbmp;
	BMPINFOHEADER bmih;
	BMPCOREHEADER bmch;
   BMPFILEHEADER bmfh;
	RGBTRIPLE t;
	RGBQUAD q;
   long offset,nbytes;

   // open file
   fbmp = BUF;
   if( fbmp == NULL )
		return 0;

   // read header and check
   memcpy(&bmfh,fbmp,sizeof(BMPFILEHEADER));
   fbmp+=sizeof(BMPFILEHEADER);

   if( bmfh.bfType != BMP_SIGNATURE_WORD ) return 0;
	bmpHdr->dspColors=QueryColors()+1;
   bmpHdr->sigWord=bmfh.bfType;
   bmpHdr->imgOffset=bmfh.bfOffBits;
   bmpHdr->bmpMWIObuf=NULL;

   // determine which format version is used
   // from structure size field
   nbytes = 0;
   memcpy(&nbytes,fbmp,sizeof(long));

   // read bitmap header and palette

   // pre Windows 3.0 version
   if( nbytes == sizeof(BMPCOREHEADER) ){
		//DebugStr(1,10,MC_YRes-20,"pre Windows 3.0 version              "); 
		memcpy(&bmch,fbmp,sizeof(BMPCOREHEADER));
		fbmp+=sizeof(BMPCOREHEADER);

		bmpHdr->width   = bmch.bcWidth;
		bmpHdr->height  = bmch.bcHeight;
		bmpHdr->bits	 = bmch.bcBitCount;
		bmpHdr->planes  = bmch.bcPlanes;
		bmpHdr->rle     = 0;
		bmpHdr->bmpColors  = 1 << bmpHdr->bmpColors;
      bmpHdr->xRes	 = 0;
      bmpHdr->yRes	 = 0;
      
		if( bmpHdr->bmpColors > 0 ){
			for(  i=0; i<bmpHdr->bmpColors; i++ ){
				memcpy(&t,fbmp,sizeof(RGBTRIPLE));
				fbmp+=sizeof(BMPCOREHEADER);

				imgColors[i].palRed    = t.red;
				imgColors[i].palGreen  = t.green;
				imgColors[i].palBlue   = t.blue;
				imgColors[i].palRed   *= 256;
				imgColors[i].palGreen *= 256;
				imgColors[i].palBlue  *= 256;
			}
		}
	}

   // Windows 3.0 and later version
   else if( nbytes == sizeof(BMPINFOHEADER) ){
	//DebugStr(1,10,MC_YRes-20,"Windows 3.0 and later version              ");
		memcpy(&bmih,fbmp,sizeof(BMPINFOHEADER));
		fbmp+=sizeof(BMPINFOHEADER);

		bmpHdr->width   = (int) bmih.biWidth;
		bmpHdr->height  = (int) bmih.biHeight;
		bmpHdr->bits    = bmih.biBitCount; 
		bmpHdr->planes  = bmih.biPlanes;
		bmpHdr->rle     = (int) bmih.biCompression;
		bmpHdr->bmpColors  = 2;
		bmpHdr->imgSize = bmih.biSizeImage;
		bmpHdr->xRes	 = bmih.biXPelsPerMeter;
		bmpHdr->yRes	 = bmih.biYPelsPerMeter; 
		bmpHdr->rowBytes= bmpHdr->bits*bmpHdr->width;
		
		switch(bmpHdr->bits){
			case 1: 
				bmpHdr->bmpColors=2;  
				break;
			case 4: 
				bmpHdr->bmpColors=16;
				break;
			case 8: 
				bmpHdr->bmpColors=256; 
				break;
		}
		if(imgColors){
			if( bmpHdr->bmpColors > 0 ){
				for( i=0; i<bmpHdr->bmpColors; i++ ){
					memcpy(&q,fbmp,sizeof(RGBQUAD));
					fbmp+=sizeof(RGBQUAD);
					imgColors[i].palRed    = q.red;
					imgColors[i].palGreen  = q.green;
					imgColors[i].palBlue   = q.blue;
					imgColors[i].palRed   *= 256;
					imgColors[i].palGreen *= 256;
					imgColors[i].palBlue  *= 256; 
				} //end of for loop
			}//end of ncolors>0 
		}//end of imgColors
	}//end of else if

   // unknown format
	else return 0;
	return 1;
}

int SEQ GetBMP(void *BUF, rect *SOURCER, rect *DESTR)
{
	register int j,i;
	char *fbmp;
   BMPHEADER bmpHeader;
   palData RGBCOLORS[256];
	imageHeader    *imPtr;
	image *buf2,*pkPtr,*buf,*scanln;
	int colortable=0,err,maxcolors;
	rect sR, dR;
	int rwbytes;
	grafPort *imagePort;
   long offset,nbytes;
   
	GetPort(&imagePort);
	if(!GetBMPInfo(BUF,&bmpHeader,RGBCOLORS)) return 0;    
   
	if(bmpHeader.dspColors <=16 && bmpHeader.bmpColors >16) return 0;

   if(SOURCER)
   	DupRect(SOURCER,&sR);
   else SetRect(&sR,0,0,(int)bmpHeader.width,1);
	
   sR.Ymax =sR.Ymin+bmpHeader.height;
   sR.Ymin =sR.Ymax-1;


   if(DESTR)
   	DupRect(DESTR,&dR);
   else DupRect(&imagePort->portRect,&dR);

   dR.Ymax =dR.Ymin+bmpHeader.height;
   dR.Ymin =dR.Ymax-1;
   	
	meta_hd_size=(sizeof(imageHeader));

		
   // position to start of bitmap
   fbmp= (char *)BUF+bmpHeader.imgOffset;;

	scanln = (image *)malloc((unsigned int)bmpHeader.width*5);
	if( scanln == NULL ) return 0;

	buf = (image *)malloc((unsigned int)bmpHeader.width*5);
	if( buf == NULL ) {
		free(scanln);
		return 0;
	}
   buf2 = (image *)malloc((unsigned int)bmpHeader.width*5);
	if( buf2 == NULL ) {
		free(scanln);
		free(buf);
		return 0;
	}

   if( ( pkPtr = (Byte *)( malloc( (unsigned int) rwbytes ) ) )  == NULL ){    
	   free(pkPtr); 
		free(scanln);
		free(buf);
		free(buf2);
		return 0;
	}

   rle_method = bmpHeader.rle;
   pxl_size   = bmpHeader.bits;
	bmp_buf    =fbmp;
		
   ReadImage(&sR,buf);
   imPtr = (imageHeader *)pkPtr;       // pointer to MW image header
   imPtr->imWidth    = bmpHeader.width;
   imPtr->imHeight   = 1;
   imPtr->imAlign    = 0;
   imPtr->imBits     = 8;
   imPtr->imPlanes   = 1;
   imPtr->imRowBytes = rwbytes; //can be 24 bits per pixel.. 3 planes? 

	if(bmpHeader.dspColors>16){
	   for( i=bmpHeader.height; i>0; i-- ){
			BmpBUFDecode( buf, bmpHeader.width );
			for(j=0;j<bmpHeader.width;j++)
				pkPtr[j+meta_hd_size]=buf[j];     //add imageHeader (12 bytes) for WriteImage buf record
			memset(buf2,0x00,sizeof(buf2));
			
			XlateImage(pkPtr,buf2,8,1,NULL);
			WriteImage(&sR,buf2);
			sR.Ymin -=1;
			sR.Ymax -=1;
		}
	}
	else{
	   for( i=bmpHeader.height; i>0; i-- ){
			BmpBUFDecode( buf, bmpHeader.width );
			for(j=0;j<bmpHeader.width;j++)
				pkPtr[j+meta_hd_size]=buf[j];     //add imageHeader (12 bytes) for WriteImage buf record
			memset(buf2,0x00,sizeof(buf2));
			
			XlateImage(pkPtr,buf2,1,4,NULL);
			WriteImage(&dR,buf2);
			dR.Ymin -=1;
			dR.Ymax -=1;
		}
	}
	BmpTerm();

	free(buf2);
	free(pkPtr);
	free(scanln);
	free(buf);
   return 1;
}


//..................class BmpDecoder



BmpDecoder( FILE *f_in, int rle, int nbits )
{
   cond = xOKAY;
   rle_method = rle;
   pxl_size = nbits;
   bmp_file = f_in;
}

int BmpStatus( void )
{
   if( ferror(bmp_file) ) cond = xIOERROR;
   if( feof(bmp_file) ) cond = xOUTOFSYNC;
   return cond;
}

int BmpInit( void )
{
   return BmpStatus();
}

int BmpTerm( void )
{
   return BmpStatus();
}

void BmpRead_none( unsigned char *buf, int npxls )
{
   int i, j, nbytes,x,m;
   int p1,p2;
	switch( pxl_size ){
		case 1 :
			nbytes = (npxls + 7) / 8;
			nbytes = (nbytes + 3) / 4;
			nbytes *= 4;
			while( npxls > 0 ){
				nbytes--;
				x = fgetc( bmp_file );
				m = 0x80;
				while( (npxls > 0) && (m != 0) ){
					*buf++ = (x & m) ? 1 : 0;
					m >>= 1;
               npxls--;
            }
         }
			while( nbytes-- > 0 )
				fgetc( bmp_file );
         break;

      case 4 :
			nbytes = (npxls + 1) / 2;
			nbytes = (nbytes + 3) / 4;
			nbytes *= 4;
			while( npxls > 0 ){
				nbytes--;
				x = fgetc( bmp_file );
				p2 = x & 0x0F;
				p1 = (x>>4) & 0x0F;
				*buf++ = p1;
				npxls--;
				*buf++ = p2;
				npxls--;

			}
			while( nbytes-- > 0 )
			fgetc( bmp_file );
			break;

      case 8 :
         nbytes = (npxls + 3) / 4;
         nbytes *= 4;
         fread( buf, npxls, 1, bmp_file );
			nbytes -= npxls;
			while( nbytes-- > 0 )
				fgetc( bmp_file );
         break;

      case 24 :
			nbytes = (npxls*3 + 3) / 4;
			nbytes *= 4;
			fread( buf, npxls, 3, bmp_file );
			nbytes -= npxls*3;
			while( nbytes-- > 0 )
				fgetc( bmp_file );
			// reorder bgr to rgb
			for( i=0, j=0; i<npxls; i++, j+=3 ){
				int k = buf[j];
				buf[j] = buf[j+2];
				buf[j+2] = k;
         }
	   break;
   }
}

void BmpRead_rle4( unsigned char *buf, int npxls )
{
	int b1, b2;
	while( cond == xOKAY ){
      b1 = fgetc( bmp_file );
		b2 = fgetc( bmp_file );
		if( b1 == 0 ){  //.... absolute or escape
	   	switch( b2 ){
				case 0 :  //.... end-of-line
               return;

            case 1 :  //.... end-of-image
               cond = xENDOFIMAGE;
               return;

				case 2 :  //.... delta
				   cond = xUNSUPPORTED;
		         return;
		
				default : //.... nonrepeating run
               while( b2-- ){
                  if( npxls > 0 ){
                     *buf++ = fgetc( bmp_file );
                     npxls--;
                  }
               }
		   		break;
         }// end switch
      }// end if(b1 == 0)
      else{  //............. encoded
         unsigned char p1 = b2      & 0x0F;
		   unsigned char p2 = (b2>>4) & 0x0F;
		   unsigned char p;
	      if( b1 & 1 ) { p=p1; p1=p2; p2=p; }
         while( b1-- ){
            if( npxls > 0 ){
               *buf++ = (b1 & 1 ) ? p1 : p2;
		   		npxls--;
				}
         }// end while b1--
		}// end else
   }// end while cond
}

void BmpRead_rle8( unsigned char *buf, int npxls )
{
   int b1, b2;
   
   while( cond == xOKAY ){
		b1 = fgetc( bmp_file );
		b2 = fgetc( bmp_file );
      if( b1 == 0 ){  //.... absolute or escape
			switch( b2 ){
            case 0 :  //.... end-of-line
			   	return;

				case 1 :  //.... end-of-image
               cond = xENDOFIMAGE;
               return;

            case 2 :  //.... delta
				   cond = xUNSUPPORTED;
               return;

				default : //.... nonrepeating run
               while( b2-- ){
						if( npxls > 0 ){
                     *buf++ = fgetc( bmp_file );
                     npxls--;
                  }
               }
				   break;
         }
      }
		else{  //............. encoded
         while( b1-- ){
            if( npxls > 0 ){
               *buf++ = b2;
               npxls--;
            }
	   	}
      }
   }
}

int BmpDecode( unsigned char *buf, int npxls )
{
   switch( rle_method )
   {
      case xBI_NONE :
			BmpRead_none( buf, npxls );
			break;

      case xBI_RLE8 :
			BmpRead_rle8( buf, npxls );
			break;

      case xBI_RLE4 :
			BmpRead_rle4( buf, npxls );
			break;

      default :
			cond = xBADPARAM;
			break;
   }

   return BmpStatus();
}

//..................class BmpEncoder

BmpEncoder( FILE *f_out, int nbits )
{
   pxl_size = nbits;
   bmp_file = f_out;
}

void BmpWrt_line( unsigned char *buf, int npxls )
{
   int i, j, nbytes, x, m, k;
   switch( pxl_size )
   {
      case 1 :
         nbytes = (npxls + 7) / 8;
         nbytes = (nbytes + 3) / 4;
         nbytes *= 4;
		   x = 0;
		   m = 0x80;
		   while( npxls-- > 0 ){
            if( *buf++ )
               x |= m;
            if( (m>>=1) == 0 ){
				   fputc( x, bmp_file );
				   x = 0;
               m = 0x80;
               nbytes--;
            }
         }
		   if( m != 0x80 ){
            fputc( x, bmp_file );
            nbytes--;
         }
         while( nbytes-- > 0 )
				fputc( 0, bmp_file );
		   break;

      case 4 :
         nbytes = (npxls + 1) / 2;
         nbytes = (nbytes + 3) / 4;
         nbytes *= 4;
         x = 0;
         while( npxls > 0 ){
            x = (*buf++) << 4;
            if( npxls > 1 )
               x |= *buf++;
				fputc( x, bmp_file );
				npxls -= 2;
            nbytes--;
         }
         while( nbytes-- > 0 )
            fputc( 0, bmp_file );
         break;

		case 8 :
         nbytes = (npxls + 3) / 4;
         nbytes *= 4;
         fwrite( buf, npxls, 1, bmp_file );
         nbytes -= npxls;
         while( nbytes-- > 0 )
				fputc( 0, bmp_file );
         break;
                                  
                                  
      case 24 :
         // reorder rgb to bgr
		   for( i=0, j=0; i<npxls; i++, j+=3 ){
				k = buf[j];
            buf[j] = buf[j+2];
            buf[j+2] = k;
         }
         nbytes = (npxls*3 + 3) / 4;
		   nbytes *= 4;
		   fwrite( buf, npxls, 3, bmp_file );
		   nbytes -= npxls*3;
         while( nbytes-- > 0 )
            fputc( 0, bmp_file );
         break;
   }
}


// buffer decoder for GetPCX 
int BmpBUFDecode( unsigned char *buf, int npxls )
{
   switch( rle_method )
   {
      case xBI_NONE :
           BmpBUFRead_none( buf, npxls );
	     break;

      case xBI_RLE8 :
           BmpBUFRead_rle8( buf, npxls );
           break;

      case xBI_RLE4 :
	     BmpBUFRead_rle4( buf, npxls );
	     break;

      default :
	     cond = xBADPARAM;
	     break;
   }

   return BmpStatus();
} 

void BmpBUFRead_none( unsigned char *buf, int npxls )
{
   int i, j, nbytes,x,m;
   int p1,p2;
	switch( pxl_size ){
		case 1 :
			nbytes = (npxls + 7) / 8;
			nbytes = (nbytes + 3) / 4;
			nbytes *= 4;
			while( npxls > 0 ){
				nbytes--;
				x = *bmp_buf++;
				m = 0x80;
				while( (npxls > 0) && (m != 0) ){
					*buf++ = (x & m) ? 1 : 0;
					m >>= 1;
               npxls--;
            }
         }
			while( nbytes-- > 0 )
				bmp_buf++;
         break;

      case 4 :
			nbytes = (npxls + 1) / 2;
			nbytes = (nbytes + 3) / 4;
			nbytes *= 4;
			while( npxls > 0 ){
				nbytes--;
				x = *bmp_buf++;
				p2 = x & 0x0F;
				p1 = (x>>4) & 0x0F;
				*buf++ = p1;
				npxls--;
				*buf++ = p2;
				npxls--;
			}
			while( nbytes-- > 0 )
				bmp_buf++;
			break;

      case 8 :
         nbytes = (npxls + 3) / 4;
         nbytes *= 4;
         memcpy(buf,bmp_buf,npxls);
         bmp_buf+=npxls;
			nbytes -= npxls;
			while( nbytes-- > 0 )
				bmp_buf++;
         break;

      case 24 :
			nbytes = (npxls*3 + 3) / 4;
			nbytes *= 4;
			memcpy(buf,bmp_buf,npxls*3);
			bmp_buf+=npxls*3;
			nbytes -= npxls*3;
			while( nbytes-- > 0 )
				bmp_buf++;
			// reorder bgr to rgb
			for( i=0, j=0; i<npxls; i++, j+=3 ){
				int k = buf[j];
				buf[j] = buf[j+2];
				buf[j+2] = k;
         }
	   break;
   }
}

void BmpBUFRead_rle4( unsigned char *buf, int npxls )
{
	int b1, b2;
	while( cond == xOKAY ){
      b1 = *bmp_buf++;
		b2 = *bmp_buf++;
		if( b1 == 0 ){  //.... absolute or escape
	   	switch( b2 ){
				case 0 :  //.... end-of-line
               return;

            case 1 :  //.... end-of-image
               cond = xENDOFIMAGE;
               return;

				case 2 :  //.... delta
				   cond = xUNSUPPORTED;
		         return;
		
				default : //.... nonrepeating run
               while( b2-- ){
                  if( npxls > 0 ){
                     *buf++ = *bmp_buf++;
                     npxls--;
                  }
               }
		   		break;
         }// end switch
      }// end if(b1 == 0)
      else{  //............. encoded
         unsigned char p1 = b2      & 0x0F;
		   unsigned char p2 = (b2>>4) & 0x0F;
		   unsigned char p;
	      if( b1 & 1 ) { p=p1; p1=p2; p2=p; }
         while( b1-- ){
            if( npxls > 0 ){
               *buf++ = (b1 & 1 ) ? p1 : p2;
		   		npxls--;
				}
         }// end while b1--
		}// end else
   }// end while cond
}

void BmpBUFRead_rle8( unsigned char *buf, int npxls )
{
   int b1, b2;
   while( cond == xOKAY )
   {
	b1 = *bmp_buf++;
	b2 = *bmp_buf++;
      if( b1 == 0 ){  //.... absolute or escape
		   switch( b2 ){
            case 0 :  //.... end-of-line
		   		return;

				case 1 :  //.... end-of-image
               cond = xENDOFIMAGE;
               return;

            case 2 :  //.... delta
		   		cond = xUNSUPPORTED;
               return;

				default : //.... nonrepeating run
               while( b2-- ){
						if( npxls > 0 ){
                     *buf++ = fgetc( bmp_file );
                     npxls--;
                  }
               }
		   		break;
         }
      }
		else{  //............. encoded
         while( b1-- ){
            if( npxls > 0 ){
               *buf++ = b2;
               npxls--;
            }
	   	}
      }
   }
}


int BmpEncode( unsigned char *buf, int npxls )
{
   BmpWrt_line( buf, npxls );
   return BmpStatus();
}

void InitBMPFileHeader(BMPFILEHEADER *bmpFileHeader )
{
   bmpFileHeader->bfType = BMP_SIGNATURE_WORD;
   bmpFileHeader->bfSize = bmpFileHeader->bfOffBits = 0;
   bmpFileHeader->bfReserved1 = bmpFileHeader->bfReserved2 = 0;
}

void InitRGBTriple(RGBTRIPLE *rgbTriple )
{
	rgbTriple->blue = rgbTriple->green = rgbTriple->red = 0;
}

void SetRGBTriple( int r, int g, int b, RGBTRIPLE *rgbTriple )
{
   rgbTriple->blue  = b;
   rgbTriple->green = g;
   rgbTriple->red   = r;
}

void InitBmpCoreHeader(BMPCOREHEADER * bmpCoreHdr )
{
	  bmpCoreHdr->bcSize = sizeof(BMPCOREHEADER);
	  bmpCoreHdr->bcWidth = bmpCoreHdr->bcHeight = 0;
	  bmpCoreHdr->bcPlanes = 1;
	  bmpCoreHdr->bcBitCount = 0;
}

void InitRGBQuad(RGBQUAD *rgbQuad )
{
   rgbQuad->blue = rgbQuad->green = rgbQuad->red = 0;
   rgbQuad->reserved = 0;
}
void SetRGBQuad( int r, int g, int b, RGBQUAD *rgbQuad )
{
   rgbQuad->blue  = b;
   rgbQuad->green = g;
   rgbQuad->red   = r;
}

void InitBMPInfoHeader(BMPINFOHEADER *bmpInfoHdr )
{
   bmpInfoHdr->biSize = sizeof(BMPINFOHEADER);
   bmpInfoHdr->biWidth = bmpInfoHdr->biHeight = 0;
   bmpInfoHdr->biPlanes = 1;
   bmpInfoHdr->biBitCount = 0;
   bmpInfoHdr->biCompression = bmpInfoHdr->biSizeImage = 0;
   bmpInfoHdr->biXPelsPerMeter = bmpInfoHdr->biYPelsPerMeter = 0;
   bmpInfoHdr->biClrUsed = bmpInfoHdr->biClrImportant = 0;
}


void far DrawStringFXY(int x, int y, char *ps, ...)
{
	char fline[255];
	va_list  ap;

	va_start(ap,ps);
	vsprintf(fline, ps, ap);
	va_end(ap);
	ClipRect(NULL);
	MoveTo(x,y);
	DrawString(fline);
}
