/**********************************************************************/
/*                                                                    */
/*        Copyright (c) 1994-1996 CCI Software                        */
/*                                                                    */
/* PROPRIETARY RIGHTS of CCI Software are involved in the subject     */
/* matter of this material.  All manufacturing, reproduction, use,    */
/* and sales rights pertaining to this subject matter are governed    */
/* by the license agreement.  The recipient of this software          */
/* implicitly accepts the terms of the license.                       */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/*    Program: McShell     Module: bmplab.h                           */
/*                                                                    */
/*--------------------------------------------------------------------*/
/*                                                                    */
/*    Description    : Definitions for BMP format and functions       */
/*                     that manipulate it                             */
/*--------------------------------------------------------------------*/
/*                                                                    */
/*    Author         : MICHAEL D. MANNING                             */
/*    developed on   : 01/17/1996                                     */
/*  last update     : 02/20/1996                                      */
/*                                                                    */
/*      COMPILER OPTIONS: MSVC++ 1.5                                  */ 
/*      /nologo /Gs /G2 /W2 /AL /Gx- /D "NDEBUG" /D "_DOS" /FR        */
/*                                                                    */
/*--------------------------------------------------------------------*/
/*                                                                    */
/*    COPYRIGHT (C) 1996, MIKE MANNING, CCI SOFTWARE                  */
/*                                                                    */
/**********************************************************************/

#ifndef _BMP_LAB_
#define _BMP_LAB_
#ifndef BMP_SIGNATURE_WORD
#define BMP_SIGNATURE_WORD 0x4D42
#endif

enum tCodecTypes
{
   tNONE,
   tRLE,
   tPACKBITS,
   tHUFFMAN,
   tGROUP3,
   tLZW,
   tGIFLZW
};

//.................status codes

enum xStatusCodes
{
   xOKAY        = 0x00,
   xENDOFIMAGE  = 0x01,
   xENDOFFILE   = 0x02,
   xIOERROR     = 0x04,
   xNOMEMORY    = 0x08,
   xOUTOFSYNC   = 0x10,
   xOVERFLOW    = 0x20,
   xBADPARAM    = 0x40,
   xUNSUPPORTED = 0x80
};

enum BiCompressionMethods
{
   xBI_NONE = 0,
   xBI_RGB  = 0,
   xBI_RLE4 = 2,
   xBI_RLE8 = 1
};

typedef struct xBMPFILEHEADER{
   unsigned short bfType;        // signature - 'BM'
   unsigned long  bfSize;        // file size in bytes
   unsigned short bfReserved1;   // 0
   unsigned short bfReserved2;   // 0
   unsigned long  bfOffBits;     // offset to bitmap  
}BMPFILEHEADER;

typedef struct xRGBTRIPLE{
   unsigned char blue;    // 0..255
   unsigned char green;   // 0..255
   unsigned char red;     // 0..255 
}RGBTRIPLE;

typedef struct xBMPCOREHEADER{
   unsigned long  bcSize;      // size of this struct
   short          bcWidth;     // bitmap width in pixels
   short          bcHeight;    // bitmap height in pixels
   unsigned short bcPlanes;    // num planes - always 1
   unsigned short bcBitCount;  // bits per pixel 
}BMPCOREHEADER;

typedef struct xBMPCOREINFO{
   BMPCOREHEADER    bmciHeader;
   RGBTRIPLE      bmciColors[1];
}BMPCOREINFO;

typedef struct xRGBQUAD{
   unsigned char blue;       // 0..255
   unsigned char green;      // 0..255
   unsigned char red;        // 0..255
   unsigned char reserved;   // 0  
}RGBQUAD;

typedef struct xBMPINFOHEADER{
   unsigned long  biSize;          // size of this struct
   unsigned long  biWidth;         // bmap width in pixels
   unsigned long  biHeight;        // bmap height in pixels
   unsigned short biPlanes;        // num planes - always 1
   unsigned short biBitCount;      // bits per pixel
   unsigned long  biCompression;   // compression flag
   unsigned long  biSizeImage;     // image size in bytes
   unsigned long  biXPelsPerMeter; // horz resolution
   unsigned long  biYPelsPerMeter; // vert resolution
   unsigned long  biClrUsed;       // 0 -> color table size
   unsigned long  biClrImportant;  // important color count
}BMPINFOHEADER;

typedef struct xBMPINFO{
   BMPINFOHEADER  bmiHeader;
   RGBQUAD        bmiColors[1];
}BMPINFO;  

typedef struct mwBMPHEADER{
	unsigned short sigWord;
	unsigned short width;
	unsigned short height;
	unsigned short planes;
	unsigned short bits;
	unsigned short rle;
	unsigned long  imgSize;
	unsigned short bmpColors;
	unsigned short dspColors;
	unsigned short xRes;
	unsigned short yRes; 
	unsigned long  imgOffset; 
	long           bmpMWHandle;
	image          *bmpMWIObuf;
	unsigned short rowBytes;
}BMPHEADER;
typedef BMPHEADER bmpHead;

#ifdef __cplusplus      /* If C++, disable "mangling" */
extern "C" {
#endif

/* BMP file routines */
int      SEQ  ReadBMPInfo  (char *FNAME, BMPHEADER *BMPHDR,
							palData *RGBCOLORS );
int      SEQ  ReadBMP      (char *FNAME, rect *SOURCER,
							rect *DESTR );
int      SEQ  WriteBMP     (char *FNAME, rect *SOURCER,
							palData *RGBCOLORS );
/* BMP buffer routines */
int      SEQ  GetBMP       (void *BUF, rect *SOURCER,
							rect *DESTR );

/* Incremental BMP routines */
unsigned SEQ  OpenBMP      (char *FNAME, BMPHEADER *BMPHDR,
									 palData *RGBCOLORS );
int      SEQ  NextBMP      ( BMPHEADER *BMPHDR, image *BMPIMAGE );
void     SEQ  CloseBMP     ( BMPHEADER *BMPHDR );

/* image transformation routines */
long     SEQ  xScaleImage   (image *SRCIMAGE, image *DSTIMAGE,
							int DSTWIDTH, int DSTHEIGHT );

/*
#ifndef _PCXLAB_
#define xScaleImage ScaleImage
#endif                            
*/

#ifdef __cplusplus
}
#endif

#endif
/* End of File  -  BMPLAB.H */

